use crate::{
    constants::APP_NAME,
    paths::{data_libsurvive_path, data_monado_path, data_opencomposite_path, get_data_dir},
    profile::{
        prepare_ld_library_path, LighthouseDriver, Profile, ProfileFeature, ProfileFeatureType,
        ProfileFeatures, ProfileOvrCompatibilityModule, XRServiceType,
    },
};
use std::collections::HashMap;

pub fn survive_profile() -> Profile {
    let data_dir = get_data_dir();
    let prefix = data_dir.join("prefixes/survive_default");
    let mut environment: HashMap<String, String> = HashMap::new();
    environment.insert("XRT_JSON_LOG".into(), "1".into());
    environment.insert("XRT_COMPOSITOR_SCALE_PERCENTAGE".into(), "140".into());
    environment.insert("XRT_COMPOSITOR_COMPUTE".into(), "1".into());
    environment.insert("SURVIVE_GLOBALSCENESOLVER".into(), "0".into());
    environment.insert("SURVIVE_TIMECODE_OFFSET_MS".into(), "-6.94".into());
    environment.insert("XRT_DEBUG_GUI".into(), "1".into());
    environment.insert("XRT_CURATED_GUI".into(), "1".into());
    environment.insert("U_PACING_APP_USE_MIN_FRAME_PERIOD".into(), "1".into());
    environment.insert("LD_LIBRARY_PATH".into(), prepare_ld_library_path(&prefix));
    Profile {
        uuid: "survive-default".into(),
        name: format!("Survive - {name} Default", name = APP_NAME),
        xrservice_path: data_monado_path(),
        xrservice_type: XRServiceType::Monado,
        ovr_comp: ProfileOvrCompatibilityModule {
            path: data_opencomposite_path(),
            ..Default::default()
        },
        features: ProfileFeatures {
            libsurvive: ProfileFeature {
                feature_type: ProfileFeatureType::Libsurvive,
                enabled: true,
                path: Some(data_libsurvive_path()),
                repo: None,
                branch: None,
            },
            ..Default::default()
        },
        environment,
        prefix,
        can_be_built: true,
        editable: false,
        lighthouse_driver: LighthouseDriver::Survive,
        ..Default::default()
    }
}
